;;;; -*- Mode: Lisp; Syntax: ANSI-Common-Lisp; Base: 10 -*-
;;;; *************************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; Name:          xml-utils.lisp
;;;; Purpose:       XML utilities
;;;; Programmer:    Kevin M. Rosenberg
;;;; Date Started:  Apr 2000
;;;;
;;;; $Id: xml-utils.lisp,v 1.6 2002/12/04 16:49:23 kevin Exp $
;;;;
;;;; This file, part of KMRCL, is Copyright (c) 2002 by Kevin M. Rosenberg
;;;;
;;;; KMRCL users are granted the rights to distribute and use this software
;;;; as governed by the terms of the Lisp Lesser GNU Public License
;;;; (http://opensource.franz.com/preamble.html), also known as the LLGPL.
;;;; *************************************************************************

(in-package :kmrcl)
(declaim (optimize (speed 3) (safety 2) (compilation-speed 0) (debug 3)))


(defun wrap-with-xml (str entity)
  "Returns string of xml header along with entity tag start/end with str contents"
  (format nil "<?xml version=\"1.0\" standalone=\"yes\"?>~%~%<~a>~%~a~%</~a>~%" 
	  str entity entity))


;;; XML Extraction Functions

#|
#+allegro (require :pxml)
#+allegro
(defun parse-xml-no-ws (str)
  "Return list structure of XML string with removing whitespace strings"
  (remove-tree-if #'string-ws? (parse-xml str)))
|#

(defun positions-xml-tag-contents (tag xmlstr &optional (start-xmlstr 0) (end-xmlstr nil))
  "Returns three values: the start and end positions of contents between
 the xml tags and the position following the close of the end tag."
  (let ((done nil)
	(pos start-xmlstr)
	(taglen (length tag))
	(startpos nil)
	(endpos nil)
	(nextpos nil))
    (unless end-xmlstr
      (setq end-xmlstr (length xmlstr)))
    (while (not done)
	   (let ((bracketpos (position #\< xmlstr :start pos :end end-xmlstr)))
	     (if bracketpos
		 (let* ((starttag (1+ bracketpos))
			(endtag (+ starttag taglen)))
		   (if (and (< endtag end-xmlstr)
			    (string= tag xmlstr :start2 starttag :end2 endtag))
		       (let* ((char-after-tag (char xmlstr endtag)))
			 (declare (character char-after-tag))
			 (if (or (char= #\> char-after-tag) (char= #\space char-after-tag))
			     (progn
			       (if (char= #\> char-after-tag) 
				   (setq startpos (1+ endtag))
				 (setq startpos (1+ (position #\> xmlstr :start (1+ endtag)))))
			       (setq endpos (search (format nil "</~a>" tag) xmlstr
						    :start2 startpos :end2 end-xmlstr))
			       (setq done t)
			       (if (and startpos endpos)
				   (progn
				     (setq nextpos (+ endpos taglen 3))
				     (setq pos nextpos))
				 (setf startpos nil
				       endpos nil)))
			   (setq pos (1+ endtag))))
		     (setq pos (1+ starttag)))
		   (when (> pos end-xmlstr)
		     (setq done t)))
	       (setq done t))))
    (values startpos endpos nextpos)))


(defun xml-tag-contents (tag xmlstr &optional (start-xmlstr 0) (end-xmlstr nil))
  "Returns two values: the string between XML start and end tag 
and position of character following end tag."
  (multiple-value-bind 
      (startpos endpos nextpos) 
      (positions-xml-tag-contents tag xmlstr start-xmlstr end-xmlstr)
    (if (and startpos endpos)
	(values (subseq xmlstr startpos endpos) nextpos)
      (values nil nil))))

(defun xml-cdata (str)
  (concatenate 'string "<![CDATA[" str "]]>"))

